@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
  * Parts of this code are taken from libole2/test/test-ole.c
@@ -28,21 +28,24 @@
 #include <gsf/gsf-infile.h>
 #include <gsf/gsf-infile-msole.h>
 #include <gsf/gsf-msole-utils.h>
+#include <gsf/gsf-docprop-vector.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 
 #define BIFF_TYPES_FILE    "biff-types.h"
+#define DUMP_CONTENT
 
 typedef struct {
 	guint16 opcode;
 	char *name;
 } GENERIC_TYPE;
 
+#ifdef DUMP_CONTENT
 static GPtrArray *biff_types   = NULL;
 static void
-read_types (const char *fname, GPtrArray **types)
+read_types (char const *fname, GPtrArray **types)
 {
 	FILE *file = fopen(fname, "r");
 	unsigned char buffer[1024];
@@ -52,7 +55,7 @@ read_types (const char *fname, GPtrArray **types)
 		file = fopen (newname, "r");
 	}
 	if (!file) {
-		printf ("Can't find vital file '%s'\n", fname);
+		g_printerr ("Can't find vital file '%s'\n", fname);
 		return;
 	}
 	while (!feof(file)) {
@@ -134,12 +137,35 @@ dump_biff_stream (GsfInput *stream)
 		pos = gsf_input_tell (stream);
 	}
 }
+#endif /* DUMP_CONTENT */
 
 static void
-print_property (gpointer name, gpointer value, gpointer user_data)
+cb_print_property (char const *name, GsfDocProp const *prop)
 {
-	printf ("print_property: name  = %s\n", (char *)name);
-	printf ("                value = %s\n", g_strdup_value_contents((GValue *)value));
+	GValue const *val = gsf_doc_prop_get_val  (prop);
+	char *tmp;
+
+	if (gsf_doc_prop_get_link (prop) != NULL)
+		g_print ("prop '%s' LINKED TO  -> '%s'\n",
+			 name, gsf_doc_prop_get_link (prop));
+	else
+		g_print ("prop '%s'\n", name);
+
+	if (VAL_IS_GSF_DOCPROP_VECTOR ((GValue *)val)) {
+		GValueArray *va = gsf_value_get_docprop_varray (val);
+		unsigned i;
+
+		for (i = 0 ; i < va->n_values; i++) {
+			tmp = g_strdup_value_contents (
+				g_value_array_get_nth (va, i));
+			g_print ("\t[%u] = %s\n", i, tmp);
+			g_free (tmp);
+		}
+	} else {
+		tmp = g_strdup_value_contents (val);
+		printf ("\t= %s\n", tmp);
+		g_free (tmp);
+	}
 }
 
 static int
@@ -154,7 +180,6 @@ test (unsigned argc, char *argv[])
 	GsfInfile *infile;
 	GError    *err = NULL;
 	unsigned i, j;
-	GsfDocMetaData *meta_data;
 
 	for (i = 1 ; i < argc ; i++) {
 		fprintf( stderr, "%s\n",argv[i]);
@@ -180,7 +205,9 @@ test (unsigned argc, char *argv[])
 			g_warning ("'%s' Not an OLE file: %s", argv[i], err->message);
 			g_error_free (err);
 
+#ifdef DUMP_CONTENT
 			dump_biff_stream (input);
+#endif
 
 			g_object_unref (G_OBJECT (input));
 			continue;
@@ -199,30 +226,37 @@ test (unsigned argc, char *argv[])
 
 		stream = gsf_infile_child_by_name (infile, "\05SummaryInformation");
 		if (stream != NULL) {
-			puts ( "SummaryInfo");
-			meta_data = gsf_msole_metadata_read_real (stream, &err);
-			if (err != NULL)
-				g_warning ("'%s' error: %s", argv[i], err->message);
-			else {
-				gsf_doc_meta_data_foreach (meta_data, print_property, NULL);
-				g_object_unref (meta_data);
-			}
+			GsfDocMetaData *meta_data = gsf_doc_meta_data_new ();
 
+			puts ( "SummaryInfo");
+			err = gsf_msole_metadata_read (stream, meta_data);
+			if (err != NULL) {
+				g_warning ("'%s' error: %s", argv[i], err->message);
+				g_error_free (err);
+				err = NULL;
+			} else
+				gsf_doc_meta_data_foreach (meta_data,
+					(GHFunc) cb_print_property, NULL);
+
+			g_object_unref (meta_data);
 			g_object_unref (G_OBJECT (stream));
-			
 		}
 
 		stream = gsf_infile_child_by_name (infile, "\05DocumentSummaryInformation");
 		if (stream != NULL) {
+			GsfDocMetaData *meta_data = gsf_doc_meta_data_new ();
+
 			puts ( "DocSummaryInfo");
-			meta_data = gsf_msole_metadata_read_real (stream, &err);
-			if (err != NULL)
+			err = gsf_msole_metadata_read (stream, meta_data);
+			if (err != NULL) {
 				g_warning ("'%s' error: %s", argv[i], err->message);
-			else {
-				gsf_doc_meta_data_foreach (meta_data, print_property, NULL);
-				g_object_unref (meta_data);
-			}
-			
+				g_error_free (err);
+				err = NULL;
+			} else
+				gsf_doc_meta_data_foreach (meta_data,
+					(GHFunc) cb_print_property, NULL);
+
+			g_object_unref (meta_data);
 			g_object_unref (G_OBJECT (stream));
 		}
 
@@ -230,7 +264,9 @@ test (unsigned argc, char *argv[])
 			stream = gsf_infile_child_by_name (infile, stream_names[j]);
 			if (stream != NULL) {
 				puts (j < 3 ? "Excel97" : "Excel95");
+#ifdef DUMP_CONTENT
 				dump_biff_stream (stream);
+#endif
 				g_object_unref (G_OBJECT (stream));
 				break;
 			}
